clc; clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                          INPUTS REQUIRED
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fid = fopen('simple.neu','r');
Dim = 3;                              % CURRENTLY ONLY TESTED FOR 3D MESH!!
unit= 25.4; % Gambit outputs Pts in millimeters. This for unit inches
Rlimit = 20;             % Max Radius allowed for curved side consideration              
TBC = 1;                                 % 1 if Thermal Boundary Conditions
draw = 1;                                                  % 1 to plot mesh     
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 READS NEU FILE AND CREATES REQUIRED MATRICES
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
[Npts,Nelm,Elem,ElemMidPts1,Pts,Elem1,ElmF,...
    ElmT,FaceF,FaceT,TypeF,TypeT] = readingNEU2(fid,TBC);

Pts(:,2:4)=Pts(:,2:4)/unit;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              "GAMBIT-TO-NEKTON REA" MATLAB FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
p = 1;   % Element # to Plot



% The bulk of this code is designed in organizing the data from Gambit into
% a prefered Nekton format. The format is shown below. There is no "one
% size fits all" transformation from Gambit to Nekton. Therefore each
% element is considered individually. 

%  PREFERED NEKTON FORMAT:   - edge/midpt #'s shown in ( )  
         
%                       7 ----(6)------- 6      face  corner points                                     
%                      / .             / |      1     {1,2,6,5} 
%                    (7) .           (5) |      2     {2,3,7,6}     
%                    / (11)          / (10)     3     {4,3,7,8}          
%                   8 ----(8)------ 5    |      4     {1,4,8,5}                     
%                   |    .          |    |      5     {1,2,3,4}                     
%                   |    3 .....(2).|... 2      6     {5,6,7,8}                     
%                 (12)   .         (9)  /                            
%                   | (3)           | (1)
%                   | .             | /                                                                   
%                   |.              |/                                              
%                   4 ----(4)------ 1  
                  

%getElemPts(p,Elem1,Pts,Dim);
         % getElemPts looks up element number, see's what points make up
         % that element, then searchs these points for their x,y,z values.
         % before is a matrix for element 'n' that contains the x,y,z
         % values for each point in that element. This is used in the
         % function orderElemPts described below.
%orderElem2(before);
         % The matrix from "getElemPts" is nice, but the pts within the 
         % matrix for that particular element 'e' are in no particular 
         % order. This function looks at the arbitray point order in the 
         % element and re-aranges them in the format that Nekton Prefers. 
         % This will also help with connectivity later. 
ElemInfo = orderElemPts(Nelm,Npts,Elem,Pts,Dim);
         % takes the original Elem matrix defined in the inputs and orders 
         % the point information based on the orderElem of each individual 
         % element in the Nekton format. This ensures that every element
         % has the same point order format, which ensures that each face
         % will always be made up of the same corresponding point numbers 
ElemMidPts = orderMidPts3(ElemMidPts1,ElemInfo,Nelm,Pts,Dim);
         % Takes the Element Midpoint matrix created in section 1, and
         % determines which midpoints go with the correct edge numbers
         % for each element. The output is a matrix where the first column
         % is the element number, and the remaining columns are the
         % midpoint PTS' number in edge order from edge 1 - 12                        
ElemConn = getConnectivity2(Nelm,ElemInfo);
         % Uses information from ElemInfo to determine the connectivity of
         % each element by faces. CURRENTLY ONLY TESTED FOR 3D MESH!!        
disp(' DETERMING FLUID BC           :    STARTED ****');
FluidBC  = getBC2(ElemConn,ElmF,FaceF,TypeF,Elem1,ElemInfo);
disp('                                   DONE');
        % Uses Connectiviy information and Fluid BC Inputs to determine the
        % Fluid Boundary Conditions
if TBC == 1        
disp(' DETERMING THERMAL BC         :    STARTED ****');
ThermalBC  = getBC2(ElemConn,ElmT,FaceT,TypeT,Elem1,ElemInfo);
disp('                                   DONE');
end
        % Uses Connectiviy information and Fluid BC Inputs to determine the
        % Fluid Boundary Conditions 
Curve = getCurveSides(ElemConn,ElemMidPts,Nelm,Pts,ElemInfo,Rlimit);
        % Determines curved side information. It does not include interior
        % edges, or straight edges. This matrix is used in the REA code
        % below to write the curved side section. You can set the threshold
        % for straight edges by setting "Rlimit" in the inputs section.
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     Writes "Core" of the .REA file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(' ================================================================')
disp('                       WRITING REA FILE                          ')
disp(' ================================================================')
fid = fopen('mesh.dat','w');
format short
%
%   OUTPUT ELEMENT INFO
%
disp(' ')
disp('ELEMENT INFO')
fprintf(fid,' **MESH DATA** 6 lines are X,Y,Z;X,Y,Z. Columns corners 1-4;5-8\n');
fprintf(fid,'%10i%10i%10i NEL,NDIM,NELV \n',Nelm,Dim,Nelm);
x=ones(1,4*(Dim-1));y=ones(1,4*(Dim-1));z=ones(1,4*(Dim-1));
for e=1:Nelm;
  
    for i=1:4*(Dim-1)
        x(:,i)=Pts(ElemInfo(e,i+1),2);
        y(:,i)=Pts(ElemInfo(e,i+1),3);
        z(:,i)=Pts(ElemInfo(e,i+1),4);
    end
    
   fprintf(fid,'      ELEMENT%11i [    1A]    GROUP     0\n',e);
   fprintf(fid,'%11.6f%15.6f%15.6f%15.6f\n',x(1,1:4));
   fprintf(fid,'%11.6f%15.6f%15.6f%15.6f\n',y(1,1:4));
   fprintf(fid,'%11.6f%15.6f%15.6f%15.6f\n',z(1,1:4));
   fprintf(fid,'%11.6f%15.6f%15.6f%15.6f\n',x(1,5:8));
   fprintf(fid,'%11.6f%15.6f%15.6f%15.6f\n',y(1,5:8));
   fprintf(fid,'%11.6f%15.6f%15.6f%15.6f\n',z(1,5:8));

end;
%
%   OUTPUT CURVE SIDE INFO
%
disp(' ')
disp('CURVE SIDE INFO')
if Curve(1,:) == 0
    n = 0;
else
    n = length(Curve(:,1));
end
fprintf(fid,'  ***** CURVED SIDE DATA ***** \n');
fprintf(fid,'%10i Curved sides follow IEDGE,IEL,CURVE(I),I=1,5, CCURVE\n',n);
if n ~= 0
    for j = 1:length(Curve(:,1))  % 3D ONLY for now
           if Nelm < 1000;
                fprintf(fid,'%3i%3i%14.6f%14.6f%14.6f%14.6f%14.6f m\n',Curve(j,:));
           else
                fprintf(fid,'%2i%6i%14.6f%14.6f%14.6f%14.6f%14.6f m\n',Curve(j,:));
           end
    end
end
disp(n)
%
%   OUTPUT BC INFO
%
disp('BOUNDARY CONDITION INFO')
fprintf(fid,'  ***** BOUNDARY CONDITIONS ***** \n');
fprintf(fid,'  ***** FLUID BOUNDARY CONDITIONS ***** \n');

ElemConn(:,5:7) = 0;
C = ElemConn;
for i=1:length(ElemConn(:,1))
   if Nelm < 1000
       fprintf(fid,'%2c%5i%3i%10.5f%14.5f%14.5f%14.5f%14.5f\n',FluidBC(i),...
           C(i,1),C(i,2),C(i,3),C(i,4),C(i,5),C(i,6),C(i,7));
   else
       fprintf(fid,'%2c%8i%3i%10i%14.5f%14.5f%14.5f%14.5f\n',FluidBC(i),...
           C(i,1),C(i,2),C(i,3),C(i,4),C(i,5),C(i,6),C(i,7));
   end
end
if TBC == 1
fprintf(fid,'  ***** THERMAL BOUNDARY CONDITIONS ***** \n');

for i=1:length(ElemConn(:,1))
   if Nelm < 1000
       fprintf(fid,'%2c%5i%3i%10.5f%14.5f%14.5f%14.5f%14.5f\n',ThermalBC(i),...
           C(i,1),C(i,2),C(i,3),C(i,4),C(i,5),C(i,6),C(i,7));
   else
       fprintf(fid,'%2c%8i%3i%10i%14.5f%14.5f%14.5f%14.5f\n',ThermalBC(i),...
           C(i,1),C(i,2),C(i,3),C(i,4),C(i,5),C(i,6),C(i,7));
   end
end
end
fclose(fid);
disp(' ')
disp(' ======================================  FINISHED')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                          FOR PLOTTING PURPOSES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
graph(p,Elem,Pts,Dim,ElemMidPts)

if draw == 1
    for e=1:Nelm
        ElemPts = getElemPts(e,ElemInfo,Pts,Dim);
        g = [1 2 3 4 1 5 6 2 6 7 3 7 8 4 8 5];
        for i=1:length(g)
            X(i) =ElemPts(1,g(i));
            Y(i) =ElemPts(2,g(i));
            Z(i) =ElemPts(3,g(i));
        end
         plot3(X,Y,Z,'-b'),hold on,
         xlabel 'x',ylabel 'y',zlabel 'z'
         title 'MIDPTS NOT SHOWN , ONLY ELEMENTS (coaser than it actually is)'
         axis equal
    end
end
% =========================================================================
